#include <ntddk.h>
#include <windef.h>
#include "LDasm.h"
#include "FakeFunc.h"

#define NT_DEVICE_NAME                L"\\Device\\HookPsLookupXXX"
#define DOS_DEVICE_NAME               L"\\DosDevices\\HookPsLookupXXX"

#define IOCTL_Hook				CTL_CODE(FILE_DEVICE_UNKNOWN, 0x1, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_UnHook			CTL_CODE(FILE_DEVICE_UNKNOWN, 0x2, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SetPID			CTL_CODE(FILE_DEVICE_UNKNOWN, 0x3, METHOD_BUFFERED, FILE_ANY_ACCESS)

KIRQL f_oldirql;
KSPIN_LOCK f_spinlock;

void Hook(PVOID Func,PVOID NewFunc,PVOID ProxyFunc)
{
	PMDL pMdl;
	ULONG PatchSize;
	BYTE g_HookCode[5]={0xE9,0,0,0,0};//Relative Jump
	BYTE Jmp_Orig_Code[7]={0xEA,0,0,0,0,0x08,0x00 }; //Absolute Address Jump
	PatchSize=GetPatchSize(Func,5);//Get The Number Of Bytes To Patch
	RtlCopyMemory((PBYTE)ProxyFunc,(PBYTE)Func,PatchSize);//Set The Head Of Original Function
	*((PULONG)(Jmp_Orig_Code+1))=(ULONG)((PBYTE)Func+PatchSize );//Address Of Original Function + N
	RtlCopyMemory((PBYTE)ProxyFunc+PatchSize,Jmp_Orig_Code,7);//Absolute Jump
	*((ULONG*)(g_HookCode+1))=(ULONG)NewFunc-(ULONG)Func-5;//Calculate The Relative Jump
	pMdl=IoAllocateMdl(Func,5,FALSE,FALSE,NULL);//Allocate MDL
	if(pMdl)
	{
		__try
		{
			//Lock Page
			MmProbeAndLockPages(pMdl,KernelMode,IoWriteAccess);
		}
		__except(EXCEPTION_EXECUTE_HANDLER)
		{
			IoFreeMdl(pMdl);
		}
		//Raise The Interrupt Request Level
		f_oldirql=KeRaiseIrqlToDpcLevel();
		//Initialize And Acquire Spinlock
		KeInitializeSpinLock(&f_spinlock);
		KeAcquireSpinLockAtDpcLevel(&f_spinlock);
		//Close Write-Protection
		_asm
		{
			cli              
			mov eax, cr0   
			and eax, not 10000h
			mov cr0, eax     
		}
		//Hook Operation
		RtlCopyMemory(Func, g_HookCode, 5);
		//Reopen Write-Protection
		_asm 
		{
			mov eax, cr0    
			or  eax, 10000h            
			mov cr0, eax              
			sti                  
		}
		//Release Spinlock
		KeReleaseSpinLockFromDpcLevel(&f_spinlock);
		//Lower Interupt Request Level
		KeLowerIrql(f_oldirql);
		//Unlock Pages
		MmUnlockPages(pMdl);
		//Free MDL
		IoFreeMdl(pMdl);
	}
}

void UnHook(PVOID Func,PVOID ProxyFunc)
{
	PMDL pMdl;
	//Allocate MDL
	pMdl=IoAllocateMdl(Func,5,FALSE,FALSE,NULL);
	if(pMdl)
	{
		__try
		{
			//Lock Pages
			MmProbeAndLockPages(pMdl,KernelMode,IoWriteAccess);
		}
		__except(EXCEPTION_EXECUTE_HANDLER)
		{
			IoFreeMdl(pMdl);
		}
		//Raise Interupt Request Level
		f_oldirql=KeRaiseIrqlToDpcLevel();
		//Initialize And Acquire Spinlock
		KeInitializeSpinLock(&f_spinlock);
		KeAcquireSpinLockAtDpcLevel(&f_spinlock);
		//Close Write-Protection
		_asm
		{
			cli              
			mov eax, cr0   
			and eax, not 10000h
			mov cr0, eax     
		}
		//Unhook Operation
		RtlCopyMemory(Func,ProxyFunc,5);
		//Reopen Write-Protection
		_asm 
		{
			mov eax, cr0    
			or  eax, 10000h            
			mov cr0, eax              
			sti                  
		}
		//Release Spinlock
		KeReleaseSpinLockFromDpcLevel(&f_spinlock);
		//Lower Interupt Request Level
		KeLowerIrql(f_oldirql);
		//Unlock Pages
		MmUnlockPages(pMdl);
		//Free MDL
		IoFreeMdl(pMdl);
	}
}

void StartHook()
{
	//deal with the NTKERNELAPI
	UNICODE_STRING uniFuncName1;
	UNICODE_STRING uniFuncName2;
	RtlInitUnicodeString(&uniFuncName1,L"PsLookupProcessByProcessId");
	RtlInitUnicodeString(&uniFuncName2,L"PsLookupThreadByThreadId");
	PsLookupProcessByProcessId=MmGetSystemRoutineAddress(&uniFuncName1);
	PsLookupThreadByThreadId=MmGetSystemRoutineAddress(&uniFuncName2);
	//Initialize Proxy Function
	Old_PsLookupProcessByProcessId=ExAllocatePool(NonPagedPool,20);
	Old_PsLookupThreadByThreadId=ExAllocatePool(NonPagedPool,20);
	//NOP The Proxy Function
	memset(Old_PsLookupProcessByProcessId,0x90,20);
	memset(Old_PsLookupThreadByThreadId,0x90,20);
	//Start To Hook
	Hook(PsLookupProcessByProcessId,fake_PsLookupProcessByProcessId,Old_PsLookupProcessByProcessId);
	Hook(PsLookupThreadByThreadId,fake_PsLookupThreadByThreadId,Old_PsLookupThreadByThreadId);
}

void StopHook()
{
	//Unhook
	UnHook(PsLookupProcessByProcessId,Old_PsLookupProcessByProcessId);
	UnHook(PsLookupThreadByThreadId,Old_PsLookupThreadByThreadId);
	//Release the memory
	ExFreePool(Old_PsLookupProcessByProcessId);
	ExFreePool(Old_PsLookupThreadByThreadId);
}